@@ -1,7 +1,7 @@
-import { AsyncIterableX, empty, from } from 'ix/asynciterable';
+import { AsyncIterableX, from } from 'ix/asynciterable';
+import { empty } from 'ix/asynciterable/empty';
 import { merge } from 'ix/asynciterable/merge';
 import { catchError, concatMap, withAbort } from 'ix/asynciterable/operators';
-import { isEmpty } from 'lodash';
 
 import {
   DataSourceRuleGroupIdentifier,
@@ -15,14 +15,14 @@ import {
   PromRuleGroupDTO,
 } from 'app/types/unified-alerting-dto';
 
-import { usePopulateGrafanaPrometheusApiCache } from '../../api/prometheusApi';
-import { RulesFilter } from '../../search/rulesSearchParser';
+import { RuleSource, RulesFilter } from '../../search/rulesSearchParser';
 import {
   getDataSourceByUid,
   getDatasourceAPIUid,
   getExternalRulesSources,
   isSupportedExternalRulesSourceType,
 } from '../../utils/datasource';
+import { RulePositionHash, createRulePositionHash } from '../rulePositionHash';
 
 import { groupFilter, ruleFilter } from './filters';
 import { useGrafanaGroupsGenerator, usePrometheusGroupsGenerator } from './prometheusGroupsGenerator';
@@ -44,6 +44,22 @@ export interface PromRuleWithOrigin {
   rule: PromRuleDTO;
   groupIdentifier: DataSourceRuleGroupIdentifier;
   origin: 'datasource';
+  /**
+   * Position hash encoding both the rule's index and the total number of rules in the group.
+   * Format: "<index>:<totalRules>" (e.g., "0:3", "1:5")
+   *
+   * This is used as a tiebreaker when multiple identical rules exist in a group and need to be
+   * matched against their counterparts in another rule source (e.g., matching Prometheus rules
+   * to Ruler API rules). The hash ensures that identical rules are matched by their position
+   * only when both groups have the same structure (same number of rules).
+   *
+   * @example
+   * // Two identical alerts in different positions
+   * Rule at position 0 in a 3-rule group: rulePositionHash = "0:3"
+   * Rule at position 1 in a 3-rule group: rulePositionHash = "1:3"
+   * // These won't match rules in a 2-rule group (e.g., "0:2") even if identical
+   */
+  rulePositionHash: RulePositionHash;
 }
 
 interface GetIteratorResult {
@@ -52,7 +68,6 @@ interface GetIteratorResult {
 }
 
 export function useFilteredRulesIteratorProvider() {
-  const { populateGroupResponseCache } = usePopulateGrafanaPrometheusApiCache();
   const allExternalRulesSources = getExternalRulesSources();
 
   const prometheusGroupsGenerator = usePrometheusGroupsGenerator();
@@ -65,7 +80,7 @@ export function useFilteredRulesIteratorProvider() {
     const normalizedFilterState = normalizeFilterState(filterState);
     const hasDataSourceFilterActive = Boolean(filterState.dataSourceNames.length);
 
-    const grafanaRulesGenerator = from(
+    const grafanaRulesGenerator: AsyncIterableX<RuleWithOrigin> = from(
       grafanaGroupsGenerator(groupLimit, {
         contactPoint: filterState.contactPoint ?? undefined,
         health: filterState.ruleHealth ? [filterState.ruleHealth] : [],
@@ -76,12 +91,9 @@ export function useFilteredRulesIteratorProvider() {
       concatMap((groups) =>
         groups
           .filter((group) => groupFilter(group, normalizedFilterState))
-          .flatMap((group) => {
-            populateGroupResponseCache(group);
-            return group.rules.map((rule) => [group, rule] as const);
-          })
-          .filter(([, rule]) => ruleFilter(rule, normalizedFilterState))
-          .map(([group, rule]) => mapGrafanaRuleToRuleWithOrigin(group, rule))
+          .flatMap((group) => group.rules.map((rule) => ({ group, rule })))
+          .filter(({ rule }) => ruleFilter(rule, normalizedFilterState))
+          .map(({ group, rule }) => mapGrafanaRuleToRuleWithOrigin(group, rule))
       ),
       catchError(() => empty())
     );
@@ -91,38 +103,57 @@ export function useFilteredRulesIteratorProvider() {
       ? getRulesSourcesFromFilter(filterState)
       : allExternalRulesSources;
 
-    // If no data sources, just return Grafana rules
-    if (isEmpty(externalRulesSourcesToFetchFrom)) {
+    if (filterState.ruleSource === RuleSource.Grafana) {
       return { iterable: grafanaRulesGenerator, abortController };
     }
 
-    // Create a generator for each data source
-    const dataSourceGenerators = externalRulesSourcesToFetchFrom.map((dataSourceIdentifier) => {
-      const promGroupsGenerator = from(prometheusGroupsGenerator(dataSourceIdentifier, groupLimit)).pipe(
-        withAbort(abortController.signal),
-        concatMap((groups) =>
-          groups
-            .filter((group) => groupFilter(group, normalizedFilterState))
-            .flatMap((group) => group.rules.map((rule) => [group, rule] as const))
-            .filter(([, rule]) => ruleFilter(rule, normalizedFilterState))
-            .map(([group, rule]) => mapRuleToRuleWithOrigin(dataSourceIdentifier, group, rule))
-        ),
-        catchError(() => empty())
-      );
+    const dataSourceGenerators: Array<AsyncIterableX<RuleWithOrigin>> = externalRulesSourcesToFetchFrom.map(
+      (dataSourceIdentifier) => {
+        const promGroupsGenerator: AsyncIterableX<RuleWithOrigin> = from(
+          prometheusGroupsGenerator(dataSourceIdentifier, groupLimit)
+        ).pipe(
+          withAbort(abortController.signal),
+          concatMap((groups) =>
+            groups
+              .filter((group) => groupFilter(group, normalizedFilterState))
+              .flatMap((group) => group.rules.map((rule, index) => ({ group, rule, index })))
+              .filter(({ rule }) => ruleFilter(rule, normalizedFilterState))
+              .map(({ group, rule, index }) => mapRuleToRuleWithOrigin(dataSourceIdentifier, group, rule, index))
+          ),
+          catchError(() => empty())
+        );
 
-      return promGroupsGenerator;
-    });
+        return promGroupsGenerator;
+      }
+    );
 
-    // Merge all generators
-    return {
-      iterable: merge<RuleWithOrigin>(grafanaRulesGenerator, ...dataSourceGenerators),
-      abortController,
-    };
+    const iterablesToMerge: Array<AsyncIterableX<RuleWithOrigin>> = [];
+    const includeGrafana = filterState.ruleSource !== 'datasource';
+    const includeExternal = true;
+
+    if (includeGrafana) {
+      iterablesToMerge.push(grafanaRulesGenerator);
+    }
+    if (includeExternal) {
+      iterablesToMerge.push(...dataSourceGenerators);
+    }
+
+    const iterable = mergeIterables(iterablesToMerge);
+
+    return { iterable, abortController };
   };
 
   return getFilteredRulesIterable;
 }
 
+function mergeIterables(iterables: Array<AsyncIterableX<RuleWithOrigin>>): AsyncIterableX<RuleWithOrigin> {
+  if (iterables.length === 0) {
+    return empty();
+  }
+  const [firstIterable, ...rest] = iterables;
+  return merge(firstIterable, ...rest);
+}
+
 /**
  * Finds all data sources that the user might want to filter by.
  * Only allows Prometheus and Loki data source types.
@@ -152,7 +183,8 @@ function getRulesSourcesFromFilter(filter: RulesFilter): DataSourceRulesSourceId
 function mapRuleToRuleWithOrigin(
   rulesSource: DataSourceRulesSourceIdentifier,
   group: PromRuleGroupDTO,
-  rule: PromRuleDTO
+  rule: PromRuleDTO,
+  ruleIndex: number
 ): PromRuleWithOrigin {
   return {
     rule,
@@ -163,6 +195,7 @@ function mapRuleToRuleWithOrigin(
       groupOrigin: 'datasource',
     },
     origin: 'datasource',
+    rulePositionHash: createRulePositionHash(ruleIndex, group.rules.length),
   };
 }
 
