@@ -1,6 +1,8 @@
 import { from, Observable, of } from 'rxjs';
 import { catchError } from 'rxjs/operators';
+
 import { AnnotationEvent, DataSourceApi } from '@grafana/data';
+import { shouldUseLegacyRunner } from 'app/features/annotations/standardAnnotationSupport';
 
 import { AnnotationQueryRunner, AnnotationQueryRunnerOptions } from './types';
 import { handleAnnotationQueryRunnerError } from './utils';
@@ -11,6 +13,10 @@ export class LegacyAnnotationQueryRunner implements AnnotationQueryRunner {
       return false;
     }
 
+    if (shouldUseLegacyRunner(datasource)) {
+      return true;
+    }
+
     return Boolean(datasource.annotationQuery && !datasource.annotations);
   }
 
@@ -19,8 +25,17 @@ export class LegacyAnnotationQueryRunner implements AnnotationQueryRunner {
       return of([]);
     }
 
-    return from(datasource!.annotationQuery!({ range, rangeRaw: range.raw, annotation, dashboard })).pipe(
-      catchError(handleAnnotationQueryRunnerError)
-    );
+    if (datasource?.annotationQuery === undefined) {
+      console.warn('datasource does not have an annotation query');
+      return of([]);
+    }
+
+    const annotationQuery = datasource.annotationQuery({ range, rangeRaw: range.raw, annotation, dashboard });
+    if (annotationQuery === undefined) {
+      console.warn('datasource does not have an annotation query');
+      return of([]);
+    }
+
+    return from(annotationQuery).pipe(catchError(handleAnnotationQueryRunnerError));
   }
 }
